#ifndef __HXT5G02A_H__
#define __HXT5G02A_H__

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/i2c.h>
#include <linux/i2c-proc.h>
#include <linux/init.h>

#define	HXT5G02A_SYSCTL_READ			0x1000
#define	HXT5G02A_SYSCTL_WRITE			0x1001
#define	HXTTUNER_SYSCTL_READ			0x1002
#define	HXTTUNER_SYSCTL_WRITE			0x1003

#define HXT5G02AREG_SYSTEM				0x00
#define HXT5G02AREG_AFT					0x04
#define HXT5G02A_BLK_NR					3

#define HXT5G02A_SYS_NTSCMN_SWM			0xD2
#define HXT5G02A_SYS_NTSCMN_ADJM		0x2F
#define HXT5G02A_SYS_NTSCMN_DATM		0x08

#define HXT5G02A_SYS_PALBG_SECAM_SWM	0xD6
#define HXT5G02A_SYS_PALBG_SECAM_ADJM	0x6C
#define HXT5G02A_SYS_PALBG_SECAM_DATM	0x09

#define HXT5G02A_SYS_PALI_SWM			0xD6
#define HXT5G02A_SYS_PALI_ADJM			0x6C
#define HXT5G02A_SYS_PALI_DATM			0x0A

#define HXT5G02A_SYS_PALDK_SECAM_SWM	0xD6
#define HXT5G02A_SYS_PALDK_SECAM_ADJM	0x6C
#define HXT5G02A_SYS_PALDK_SECAM_DATM	0x0B

#define HXT5G02A_SYS_SECAMLL_SWM		0xC6
#define HXT5G02A_SYS_SECAMLL_ADJM		0x6C
#define HXT5G02A_SYS_SECAMLL_DATM		0x0B

#define HXT5G02A_SYS_FM_SWM				0xCE
#define HXT5G02A_SYS_FM_ADJM			0x00
#define HXT5G02A_SYS_FM_DATM			0x08

#define HXT5G02A_SYS_MAX_STD			13

#define TCH_BW_NTSC						0x37
#define TCH_BW_PAL						0x35
#define TCH_BW_SECAM					0x35
#define TCH_BW_TBD						0x00

#define TCH_INPUT_US_AIR				0x0001
#define TCH_INPUT_US_CATV				0x0003

#define TCH_INPUT_CCIR					0x0004
#define TCH_INPUT_CCIR_CATV				0x000C

#define TCH_INPUT_FRANCE				0x0010

#define TCH_INPUT_CHINA					0x0040
#define TCH_INPUT_CHINA_CATV			0x00C0

#define TCH_INPUT_JPN					0x0100
#define TCH_INPUT_JPN_CATV				0x0300

#define TCH_INPUT_GLOBAL				0x10000
#define TCH_INPUT_GLOBAL_CATV			0x30000

#define CHNUM_US_AIR					69
#define CHNUM_US_CATV					125	

#define CHNUM_JPN_AIR					62
#define CHNUM_JPN_CATV					63	

#define HXTTUNREREG_TUNER				0x01
#define HXTTUNER_BLK_NR					6

#define	HXTTUNER_I2C_IOADDR				0x60
#define	HXT5G02A_I2C_IOADDR				0x43	// 

typedef struct _tagChannelData
{
	int	nCurrInput;
	int	nArea, nCATV;
	int	nChNum;
	int	*pChannels;
}	CHANNELDATA;

typedef CHANNELDATA *PCHANNELDATA;

typedef struct _tagHXT5G02AData 
{
	int 				nSysCtlID;
	struct semaphore	UpdateLock;
	char 				cValid;					// !=0 if following fields are valid
	unsigned long		uLastUpdated;			// In jiffies 
	unsigned short		wReg;
	unsigned char 		byRead[HXT5G02A_BLK_NR<<1], byWrite[HXT5G02A_BLK_NR<<1];	// Register values
	devfs_handle_t		DevFS;
	int					nVSTD, nOpen;
	struct i2c_client	*pClient;
} 	HXT5G02ADATA;

typedef HXT5G02ADATA *PHXT5G02ADATA;

typedef struct _tagHXTTunerData 
{
	int 				nSysCtlID;
	struct semaphore	UpdateLock;
	char 				cValid;					// !=0 if following fields are valid
	unsigned long		uLastUpdated;			// In jiffies 
	unsigned short		wReg;
	unsigned char 		byRead[HXTTUNER_BLK_NR], byWrite[HXTTUNER_BLK_NR];	// Register values
	devfs_handle_t		DevFS;
	CHANNELDATA			ChannelData;
	int					nOpen;
	struct i2c_client	*pClient;
} 	HXTTUNERDATA;

typedef HXTTUNERDATA *PHXTTUNERDATA;

typedef struct _tagTunerSystemData
{
//	union
//	{
//		struct
//		{
//			unsigned char	bySwitchMode;
//			unsigned char	byAdjustMode;
//			unsigned char	byDataMode;
//		};
		unsigned char		SystemData[3];
//	};
}	TUNERSYSTEMDATA;

typedef TUNERSYSTEMDATA *PTUNERSYSTEMDATA;

typedef struct _tagTunerChannel
{
//	union
//	{
//		struct
//		{
//			unsigned char byLO1DIV1;
//			unsigned char byLO1DIV2;
//			unsigned char byLO2DIV1;
//			unsigned char byLO2DIV2;
//			unsigned char byLO2DIV3;
//			unsigned char byBW;
//		};
		unsigned char	ChannelData[6];
//	};
	char					szFreq[8];
}	TUNERCHANNEL;

typedef TUNERCHANNEL *PTUNERCHANNEL;

#endif
